import logging
import json
import base64
from typing import Union

import httpx
from httpx import ConnectError
import asyncio

from .model import RequestsMethods, ERROR_MESSAGES, APIModel


class Api:
    """The class includes all necessary methods to make API calls to the Grafana API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def call_the_api(
        self,
        api_call: str,
        method: RequestsMethods = RequestsMethods.GET,
        json_complete: str = None,
        org_id_header: int = None,
        disable_provenance_header: bool = False,
        response_status_code: bool = False,
    ) -> any:
        """The method execute a defined API call against the Grafana endpoints

        Args:
            api_call (str): Specify the API call endpoint
            method (RequestsMethods): Specify the used method (default GET)
            json_complete (str): Specify the inserted JSON as string
            org_id_header (int): Specify the optional organization id as header for the corresponding API call
            disable_provenance_header (bool): Specify the optional disable provenance as header for the corresponding API call (default False)
            response_status_code (bool): Specify if the response should include the original status code (default False)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        api_url: str = f"{self.grafana_api_model.host}{api_call}"

        headers: dict = dict()
        if self.grafana_api_model.headers is not None:
            headers: dict = self.grafana_api_model.headers

        headers.update(
            {"Authorization": f"Bearer {self.grafana_api_model.token}"},
        )

        if (
            self.grafana_api_model.username is not None
            and self.grafana_api_model.password is not None
        ):
            credentials: str = base64.b64encode(
                str.encode(
                    f"{self.grafana_api_model.username}:{self.grafana_api_model.password}"
                )
            ).decode("utf-8")
            headers.update({"Authorization": f"Basic {credentials}"})

        headers["Content-Type"] = "application/json"
        headers["Accept"] = "application/json"

        if org_id_header is not None and isinstance(org_id_header, int):
            headers["X-Grafana-Org-Id"] = org_id_header

        if isinstance(disable_provenance_header, bool) and disable_provenance_header:
            headers["X-Disable-Provenance"] = f"{disable_provenance_header}"

        http: Union[httpx.Client, httpx.AsyncClient] = self.create_the_http_api_client(
            headers
        )

        if self.grafana_api_model.http2_support:

            async def _execute_async_api_call():
                async with http:
                    return await self._execute_the_async_api_call(
                        http, method, api_url, response_status_code, json_complete
                    )

            return asyncio.run(_execute_async_api_call())

        return self._execute_the_api_call(
            http, method, api_url, response_status_code, json_complete
        )

    def _execute_the_api_call(
        self,
        http: httpx.Client,
        method: RequestsMethods,
        api_url: str,
        response_status_code: bool,
        json_complete: str,
    ) -> any:
        """The method includes a functionality to execute a synchronous api call

        Args:
            http (httpx.Client): Specify the used synchronous client
            method (RequestsMethods): Specify the used method
            api_url (str): Specify the used api url
            response_status_code (bool): Specify if the response code should be returned
            json_complete (str): Specify the forwarded json in case of patch, post or put calls

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        try:
            if method.value == RequestsMethods.GET.value:
                return self._check_the_api_call_response(
                    http.request("GET", api_url),
                    response_status_code,
                )
            elif method.value == RequestsMethods.PUT.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        http.request("PUT", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.POST.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        http.request("POST", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.PATCH.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        http.request("PATCH", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.DELETE.value:
                return self._check_the_api_call_response(
                    http.request("DELETE", api_url), response_status_code
                )
            else:
                logging.error("Please define a valid method.")
                raise Exception
        except Exception as e:
            raise e

    async def _execute_the_async_api_call(
        self,
        http: httpx.AsyncClient,
        method: RequestsMethods,
        api_url: str,
        response_status_code: bool,
        json_complete: str,
    ):
        """The method includes a functionality to execute an asynchronous api call

        Args:
            http (httpx.AsyncClient): Specify the used asynchronous client
            method (RequestsMethods): Specify the used method
            api_url (str): Specify the used api url
            response_status_code (bool): Specify if the response code should be returned
            json_complete (str): Specify the forwarded json in case of patch, post or put calls

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        try:
            if method.value == RequestsMethods.GET.value:
                return self._check_the_api_call_response(
                    await http.request("GET", api_url),
                    response_status_code,
                )
            elif method.value == RequestsMethods.PUT.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        await http.request("PUT", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.POST.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        await http.request("POST", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.PATCH.value:
                if json_complete is not None:
                    return self._check_the_api_call_response(
                        await http.request("PATCH", api_url, content=json_complete),
                        response_status_code,
                    )
                else:
                    logging.error("Please define the json_complete.")
                    raise Exception
            elif method.value == RequestsMethods.DELETE.value:
                return self._check_the_api_call_response(
                    await http.request("DELETE", api_url), response_status_code
                )
            else:
                logging.error("Please define a valid method.")
                raise Exception
        except Exception as e:
            raise e

    @staticmethod
    def _check_the_api_call_response(
        response: any = None, response_status_code: bool = False
    ) -> any:
        """The method includes a functionality to check the output of API call method for errors

        Args:
            response (any): Specify the inserted response
            response_status_code (bool): Specify if the original status code should be attached to the result (default False)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (any): Returns the value of the api call
        """

        if Api._check_if_valid_json(response.text):
            if (
                len(json.loads(response.text)) != 0
                and type(json.loads(response.text)) == dict
            ):
                if (
                    "message" in json.loads(response.text).keys()
                    and json.loads(response.text)["message"] in ERROR_MESSAGES
                ):
                    logging.error(json.loads(response.text)["message"])
                    raise ConnectError(str(json.loads(response.text)["message"]))

            json_response: Union[dict, list] = json.loads(response.text)

            if isinstance(json_response, dict) and response_status_code:
                json_response.update({"status": response.status_code})
            elif isinstance(json_response, list) and response_status_code:
                json_response[0].update({"status": response.status_code})
            return json_response
        else:
            if response_status_code:
                return dict({"status": response.status_code, "data": response.text})
            else:
                return response

    @staticmethod
    def _check_if_valid_json(response: str) -> bool:
        """The method includes a functionality to check if the response json is valid

        Args:
            response (str): Specify the inserted response json as string

        Returns:
            result (bool): Returns if the json is valid or not
        """

        valid_json: bool = False

        if response.encode() not in [b'""\n', b"null"]:
            try:
                json.loads(response)
                valid_json = True
            except (TypeError, ValueError):
                valid_json = False

        return valid_json

    @staticmethod
    def prepare_api_string(query_string: str) -> str:
        """The method includes a functionality to prepare the api string for the queries

        Args:
            query_string (str): Specify the corresponding query string

        Returns:
            query_string (str): Returns the adjusted query string
        """

        if len(query_string) >= 1:
            return f"{query_string}&"
        else:
            return query_string

    def create_the_http_api_client(
        self, headers: dict = None
    ) -> Union[httpx.Client, httpx.AsyncClient]:
        """The method includes a functionality to create the corresponding HTTP client

        Args:
            headers (dict): Specify the optional inserted headers (Default None)

        Returns:
            client (Union[httpx.Client, httpx.AsyncClient]): Returns the corresponding client
        """

        transport: httpx.HTTPTransport = httpx.HTTPTransport(
            verify=self.grafana_api_model.ssl_context,
            retries=self.grafana_api_model.retries,
        )
        limits: httpx.Limits = httpx.Limits(
            max_connections=self.grafana_api_model.num_pools
        )
        http2: bool = self.grafana_api_model.http2_support

        if http2:
            async_transport: httpx.AsyncHTTPTransport = httpx.AsyncHTTPTransport(
                retries=self.grafana_api_model.retries, http2=http2
            )
            return httpx.AsyncClient(
                http2=True,
                limits=limits,
                timeout=self.grafana_api_model.timeout,
                headers=headers,
                transport=async_transport,
                verify=self.grafana_api_model.ssl_context,
            )
        else:
            return httpx.Client(
                limits=limits,
                timeout=self.grafana_api_model.timeout,
                headers=headers,
                transport=transport,
                verify=self.grafana_api_model.ssl_context,
            )



import json
import logging
from typing import Union

from .model import APIModel, APIEndpoints, RequestsMethods, CorrelationObject
from .api import Api


class Correlations:
    """The class includes all necessary methods to access the Grafana correlations API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_correlation(self, datasource_uid: str, correlation_uid: str) -> dict:
        """The method includes a functionality to get a specific correlation from a data source - the data source identified by source uid and the correlation uid

         Args:
            datasource_uid (str): Specify the correlation data source uid
            correlation_uid (str): Specify the correlation uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding correlation
        """

        if len(datasource_uid) != 0 and len(correlation_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{datasource_uid}/correlations/{correlation_uid}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("description") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_uid or correlation_uid defined.")
            raise ValueError

    def get_all_correlations_by_datasource_uid(self, datasource_uid: str) -> list:
        """The method includes a functionality to get all correlations from a data source - the data source identified by source uid

         Args:
            datasource_uid (str): Specify the correlation data source uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the corresponding correlations
        """

        if len(datasource_uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{datasource_uid}/correlations",
                RequestsMethods.GET,
            )

            if api_call == list() or api_call[0].get("description") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_uid defined.")
            raise ValueError

    def get_all_correlations(self) -> Union[list, dict]:
        """The method includes a functionality to get all correlations

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (Union[list, dict]): Returns the corresponding correlations
        """

        api_call: Union[list, dict] = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.DATASOURCES.value}/correlations", RequestsMethods.GET
        )

        if isinstance(api_call, dict) and (
            api_call == dict()
            or api_call.get("correlations")[0].get("description") is None
        ):
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        elif isinstance(api_call, list) and (
            api_call == list() or api_call[0].get("description") is None
        ):
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def create_correlations(self, correlation_object: CorrelationObject) -> dict:
        """The method includes a functionality to create a correlation between two data sources - the source data source identified by source uid in the path, and the target data source which is specified in the body

         Args:
            correlation_object (CorrelationObject): Specify the correlation object

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the newly created correlation
        """

        if (
            len(correlation_object.source_datasource_uid) != 0
            and len(correlation_object.target_datasource_uid) != 0
            and len(correlation_object.label) != 0
            and len(correlation_object.description) != 0
            and len(correlation_object.config_type) != 0
            and len(correlation_object.config_field) != 0
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{correlation_object.source_datasource_uid}/correlations",
                RequestsMethods.POST,
                json.dumps(
                    dict(
                        {
                            "targetUID": correlation_object.target_datasource_uid,
                            "label": correlation_object.label,
                            "description": correlation_object.description,
                            "type": correlation_object.config_type,
                            "config": {
                                "type": correlation_object.config_type,
                                "field": correlation_object.config_field,
                                "target": correlation_object.config_target,
                            },
                        }
                    )
                ),
            )

            if api_call == dict() or api_call.get("message") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "There is no datasource_uid, target_uid, label, description, config_type or config_field defined."
            )
            raise ValueError

    def delete_correlations(self, source_datasource_uid: str, correlation_uid: str):
        """The method includes a functionality to deletes a correlation

         Args:
            source_datasource_uid (str): Specify the source data source uid
            correlation_uid (str): Specify the correlation uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(source_datasource_uid) != 0 and len(correlation_uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{source_datasource_uid}/correlations/{correlation_uid}",
                RequestsMethods.DELETE,
            )

            if "Correlation deleted" != api_call.get("message"):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the correlation.")
        else:
            logging.error(
                "There is no source_datasource_uid or correlation_uid defined."
            )
            raise ValueError

    def update_correlations(
        self,
        source_datasource_uid: str,
        correlation_uid: str,
        label: str,
        description: str,
    ) -> dict:
        """The method includes a functionality to update a correlation

         Args:
            source_datasource_uid (str): Specify the source data source uid
            correlation_uid (str): Specify the correlation uid
            label (str): Specify a label for the correlation
            description (str): Specify a description for the correlation

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the updated correlation
        """

        if (
            len(source_datasource_uid) != 0
            and len(correlation_uid) != 0
            and len(label) != 0
            and len(description) != 0
        ):
            api_call: any = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{source_datasource_uid}/correlations/{correlation_uid}",
                RequestsMethods.PATCH,
                json.dumps(dict({"label": label, "description": description})),
            )

            if api_call == dict() or api_call.get("message") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "There is no source_datasource_uid, correlation_uid, label or description defined."
            )
            raise ValueError



import json
import logging
import re

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class Alerting:
    """The class includes all necessary methods to access the Grafana legacy alerting API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_alerts(
        self,
        custom_querystring: str = None,
    ) -> list:
        """The method includes a functionality to get the legacy alerts

        Args:
            custom_querystring (str): Specify the custom querystring (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of all alerts
        """

        api_string: str = ""

        if custom_querystring is not None and len(custom_querystring) != 0:
            api_string = f"{APIEndpoints.LEGACY_ALERTS.value}/{custom_querystring}"
        elif custom_querystring is None:
            api_string = APIEndpoints.LEGACY_ALERTS.value

        if len(api_string) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                api_string,
                RequestsMethods.GET,
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "Please, check the functionality. An unexpected error occurred."
            )
            raise ValueError

    def get_alerts_by_dashboard_ids(
        self,
        dashboard_ids: list,
    ) -> list:
        """The method includes a functionality to get legacy alerts specified by the dashboard ids

        Args:
            dashboard_ids (list): Specify the list of dashboard ids

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of alerts
        """

        if dashboard_ids != list():
            dashboard_ids_string: str = "?"
            for i in range(0, len(dashboard_ids)):
                dashboard_ids_string = (
                    f"{dashboard_ids_string}dashboardId={dashboard_ids[i]}"
                )

                if i < len(dashboard_ids) - 1:
                    dashboard_ids_string = f"{dashboard_ids_string}&"

            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LEGACY_ALERTS.value}/{dashboard_ids_string}",
                RequestsMethods.GET,
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard_ids defined.")
            raise ValueError

    def get_alert_by_id(self, id: int) -> dict:
        """The method includes a functionality to get the legacy alert specified by the alert id

        Args:
            id (int): Specify the id of the legacy alert

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns an alert
        """

        def _to_camel_case(input_value: str) -> str:
            content = re.findall("[A-Z][^A-Z]*", input_value)
            if content != list():
                if len(content) != 1:
                    return content[0].lower() + "".join(content[1:])
                else:
                    return content[0].lower()
            return input_value

        if id != 0:
            api_call_raw: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LEGACY_ALERTS.value}/{id}",
                RequestsMethods.GET,
            )
            api_call: dict = {_to_camel_case(k): v for k, v in api_call_raw.items()}

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def pause_alert_by_id(self, id: int, paused: bool = True):
        """The method includes a functionality to pause/ unpause a legacy alert specified by the alert id

        Args:
            id (int): Specify the id of the legacy alert
            paused (bool): Specify the pause/ unpause parameter (default True)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            json_complete: dict = {
                "paused": paused,
            }

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LEGACY_ALERTS.value}/{id}/pause",
                RequestsMethods.POST,
                json.dumps(json_complete),
            )

            if api_call.get(
                "message"
            ) != "alert paused" or "alert unpause" in api_call.get("message"):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info(api_call.get("message"))
        else:
            logging.error("There is no id defined.")
            raise ValueError



import json
import logging
from typing import Union

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    SSOSetting,
)
from .api import Api


class SSOSettings:
    """The class includes all necessary methods to access the Grafana sso settings API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_sso_settings(self) -> list:
        """The method includes a functionality to get the SSO settings for all providers

        Required Permissions:
            Action: settings:read
            Scope: settings:auth.{provider}:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the all SSO settings
        """

        api_call: Union[list, dict] = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.SSO_SETTINGS.value}",
            response_status_code=True,
        )

        status_code: int = (
            api_call[0].get("status")
            if isinstance(api_call, list)
            else api_call.get("status")
        )

        sso_settings_status_dict: dict = dict(
            {
                400: "Bad request.",
                401: "Unauthorized.",
                403: "Access Denied.",
            }
        )

        if status_code == 200:
            return api_call
        elif 400 <= status_code <= 403:
            logging.error(sso_settings_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def get_sso_settings_by_provider(self, provider: str) -> dict:
        """The method includes a functionality to get the SSO settings for the specified provider

        Args:
            provider (str): Specify the provider

        Required Permissions:
            Action: settings:read
            Scope: settings:auth.{provider}:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding provider SSO settings
        """

        if len(provider) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SSO_SETTINGS.value}/{provider}",
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            sso_settings_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                    404: "SSO Settings not found.",
                }
            )

            if status_code == 200:
                return api_call
            elif 400 <= status_code <= 404:
                logging.error(sso_settings_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no provider defined.")
            raise ValueError

    def update_sso_settings(self, provider: str, sso_setting: SSOSetting):
        """The method includes a functionality to update the SSO settings specified by the provider

         Args:
            provider (str): Specify the provider
            sso_setting (SSOSetting): Specify the SSO setting

        Required Permissions:
            Action: settings:write
            Scope: settings:auth.{provider}:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            len(provider) != 0
            and sso_setting is not None
            and len(sso_setting.api_url) != 0
            and len(sso_setting.client_id) != 0
            and len(sso_setting.client_secret) != 0
            and sso_setting.enabled is not None
            and len(sso_setting.scopes) != 0
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SSO_SETTINGS.value}/{provider}",
                RequestsMethods.PUT,
                json.dumps(
                    dict(
                        {
                            "settings": {
                                "apiUrl": sso_setting.api_url,
                                "clientId": sso_setting.client_id,
                                "clientSecret": sso_setting.client_secret,
                                "enabled": sso_setting.enabled,
                                "scopes": sso_setting.scopes,
                            }
                        }
                    )
                ),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            sso_settings_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                }
            )

            if status_code == 204:
                logging.info(
                    "You successfully updated the corresponding provider SSO setting."
                )
            elif 400 <= status_code <= 403:
                logging.error(sso_settings_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no provider or sso_settings object defined.")
            raise ValueError

    def delete_sso_settings(self, provider: str):
        """The method includes a functionality to delete the SSO settings specified by the provider

         Args:
            provider (str): Specify the provider

        Required Permissions:
            Action: settings:write
            Scope: settings:auth.{provider}:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(provider) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SSO_SETTINGS.value}/{provider}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            sso_settings_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                    404: "SSO Settings not found.",
                }
            )

            if status_code == 204:
                logging.info(
                    "You successfully deleted the corresponding provider SSO settings."
                )
            elif 400 <= status_code <= 404:
                logging.error(sso_settings_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no provider defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    TeamObject,
)
from .api import Api


class Team:
    """The class includes all necessary methods to access the Grafana team API endpoints. Be aware that all functionalities inside the class only working with the corresponding access rights, please check the following page for details: https://grafana.com/docs/grafana/latest/http_api/team/#team-api & https://grafana.com/docs/grafana/latest/developers/http_api/team_sync/#team-sync-api

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def search_team(
        self, results_per_page: int = 1000, pages: int = 1, query: str = None
    ) -> dict:
        """The method includes a functionality to get the organization teams specified by the optional pagination functionality

        Required Permissions:
            Action: teams:read
            Scope: teams:*

        Args:
            results_per_page (int): Specify the results_per_page as integer (default 1000)
            pages (int): Specify the pages as integer (default 1)
            query (str): Specify the query (default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the organization teams
        """

        api_request_url: str = (
            f"{APIEndpoints.TEAMS.value}/search?perpage={results_per_page}&page={pages}"
        )

        if query is not None and len(query) != 0:
            api_request_url: str = f"{api_request_url}&query={query}"

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            api_request_url,
        )

        if api_call == dict() or api_call.get("totalCount") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_team_by_id(self, id: int) -> dict:
        """The method includes a functionality to get the organization team specified by the id

        Required Permissions:
            Action: teams:read
            Scope: teams:*

        Args:
            id (int): Specify the id of the team

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the organization team
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}",
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def add_team(self, team: TeamObject) -> int:
        """The method includes a functionality to add an organization team specified by the TeamObject

        Required Permissions:
            Action: teams:create
            Scope: N/A

        Args:
            team (TeamObject): Specify the team

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
             team_id (int): Returns the team id
        """

        if team is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}",
                RequestsMethods.POST,
                json.dumps(dict({"name": team.name, "email": team.name})),
            )

            if api_call.get("message") != "Team created":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("teamId"))
        else:
            logging.error("There is no team defined.")
            raise ValueError

    def update_team(self, id: int, name: str, email: str):
        """The method includes a functionality to update an organization team specified by the team_id, name and email

        Required Permissions:
            Action: teams:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            name (str): Specify the team name
            email (str): Specify the team email

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and len(name) != 0 and len(email) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}",
                RequestsMethods.PUT,
                json.dumps(dict({"name": name, "email": email})),
            )

            if api_call.get("message") != "Team updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the team.")
        else:
            logging.error("There is no id, name or email defined.")
            raise ValueError

    def delete_team_by_id(self, id: int):
        """The method includes a functionality to delete an organization team specified by the team_id

        Required Permissions:
            Action: teams:delete
            Scope: teams:*

        Args:
            id (int): Specify the team id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Team deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the team.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def get_team_members(self, id: int) -> list:
        """The method includes a functionality to get all organization team users specified by the team_id

        Required Permissions:
            Action: teams.permissions:read
            Scope: teams:*

        Args:
            id (int): Specify the team id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the organization team members
        """

        if id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/members",
            )

            if api_call == list() or api_call[0].get("userId") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def add_team_member(self, id: int, user_id: int):
        """The method includes a functionality to add an organization team user specified by the team_id and the user_id

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            user_id (int): Specify the user id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and user_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/members",
                RequestsMethods.POST,
                json.dumps(dict({"userId": user_id})),
            )

            if api_call.get("message") != "Member added to Team":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully added a team member.")
        else:
            logging.error("There is no id or user_id defined.")
            raise ValueError

    def delete_team_member(self, id: int, user_id: int):
        """The method includes a functionality to delete an organization team user specified by the team_id and the user_id

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            user_id (int): Specify the user id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and user_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/members/{user_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Team Member removed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully removed a team member.")
        else:
            logging.error("There is no id or user_id defined.")
            raise ValueError

    def get_team_preferences(self, id: int) -> dict:
        """The method includes a functionality to get the organization team preferences specified by the team_id

        Required Permissions:
            Action: teams:read
            Scope: teams:*

        Args:
            id (int): Specify the team id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the organization team preferences
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/preferences",
            )

            if isinstance(api_call, dict) is False or api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def update_team_preferences(
        self,
        id: int,
        theme: str = None,
        timezone: str = None,
        home_dashboard_id: int = 0,
        home_dashboard_uid: str = None,
    ):
        """The method includes a functionality to update the organization team preferences specified by the team_id, theme, timezone and home_dashboard_id or home_dashboard_uid

        Required Permissions:
            Action: teams:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            theme (str): Specify the team theme e.g. light or dark (default Grafana None)
            timezone (str): Specify the team timezone e.g. utc or browser (default Grafana None)
            home_dashboard_id (int): Specify the home team dashboard by id (default 0)
            home_dashboard_uid (str): Specify the home team dashboard by uid (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and (
            theme is not None
            or (home_dashboard_id != 0 or home_dashboard_uid is not None)
            or timezone is not None
        ):
            team_preferences: dict = dict()

            if theme is not None:
                team_preferences.update(dict({"theme": theme}))

            if home_dashboard_id != 0 and isinstance(home_dashboard_id, int):
                team_preferences.update(dict({"homeDashboardId": home_dashboard_id}))
            else:
                team_preferences.update({"homeDashboardUID": home_dashboard_uid})

            if timezone is not None:
                team_preferences.update(dict({"timezone": timezone}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/preferences",
                RequestsMethods.PUT,
                json.dumps(team_preferences),
            )

            if api_call.get("message") != "Preferences updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the team preferences.")
        else:
            logging.error(
                "There is no id, theme, home_dashboard_id or timezone defined."
            )
            raise ValueError

    def get_external_groups(
        self,
        id: int,
    ) -> list:
        """The method includes a functionality to get the team groups specified by the team_id. The functionality is a Grafana ENTERPRISE feature

        Required Permissions:
            Action: teams.permissions:read
            Scope: teams:*

        Args:
            id (int): Specify the team id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the organization team groups
        """

        if id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/groups",
            )

            if isinstance(api_call, list) is False or api_call == list():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no team_id defined.")
            raise ValueError

    def add_external_group(
        self,
        id: int,
        team_group: str,
    ):
        """The method includes a functionality to add a group to the team specified by the team_id and the team_group. The functionality is a Grafana ENTERPRISE feature

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            team_group (str): Specify the team group

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and team_group is not None and len(team_group) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/groups",
                RequestsMethods.POST,
                json.dumps(dict({"groupId": team_group})),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            external_group_status_dict: dict = dict(
                {
                    400: "Group is already added to this team.",
                    401: "Unauthorized.",
                    403: "Permission denied.",
                    404: "Team not found.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Group added to Team":
                logging.info("You successfully added the team group.")
            elif 400 <= status_code <= 404:
                logging.error(external_group_status_dict.get(status_code))
                raise Exception
        else:
            logging.error("There is no id or team group defined.")
            raise ValueError

    def remove_external_group(
        self,
        id: int,
        team_group: str,
    ):
        """The method includes a functionality to remove a group from a team specified by the team_id and the team_group. The functionality is a Grafana ENTERPRISE feature

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Args:
            id (int): Specify the team id
            team_group (str): Specify the team group

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and team_group is not None and len(team_group) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.TEAMS.value}/{id}/groups??groupId={team_group.encode('UTF-8')}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            external_group_status_dict: dict = dict(
                {
                    401: "Unauthorized.",
                    403: "Permission denied.",
                    404: "Team not found/Group not found.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Team Group removed":
                logging.info("You successfully removed the team group.")
            elif 400 <= status_code <= 404:
                logging.error(external_group_status_dict.get(status_code))
                raise Exception
        else:
            logging.error("There is no id or team group defined.")
            raise ValueError



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class ExternalGroup:
    """The class includes all necessary methods to access the Grafana external group API endpoints.

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_external_groups(
        self,
        team_id: int,
    ) -> list:
        """The method includes a functionality to get the corresponding external groups specified by the team id

        Args:
            team_id (int): Specify the team id

        Required Permissions:
            Action: teams.permissions:read
            Scope: teams:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the external groups
        """
        if team_id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.EXTERNAL_GROUPS.value}/{team_id}/groups",
            )

            if api_call == list() or api_call[0].get("orgId") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no team_id defined.")
            raise ValueError

    def add_external_group(
        self,
        team_id: int,
        group_id: str,
    ):
        """The method includes a functionality to add the corresponding external groups specified by the team id and group id

        Args:
            team_id (int): Specify the team id
            group_id (str): Specify the group id (e.g. cn=editors,ou=groups,dc=grafana,dc=org)

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the external groups
        """
        if team_id != 0 and len(group_id) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.EXTERNAL_GROUPS.value}/{team_id}/groups",
                RequestsMethods.POST,
                json.dumps(dict({"groupId": group_id})),
            )

            if api_call.get("message") != "Group added to Team":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully added the group to the team.")
        else:
            logging.error("There is no team_id or group_id defined.")
            raise ValueError

    def remove_external_group(
        self,
        team_id: int,
        group_id: str,
    ):
        """The method includes a functionality to remove the corresponding external groups specified by the team id and group id

        Args:
            team_id (int): Specify the team id
            group_id (str): Specify the group id (e.g. cn=editors,ou=groups,dc=grafana,dc=org)

        Required Permissions:
            Action: teams.permissions:write
            Scope: teams:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the external groups
        """
        if team_id != 0 and len(group_id) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.EXTERNAL_GROUPS.value}/{team_id}/groups/{group_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Team Group removed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully removed the group from the team.")
        else:
            logging.error("There is no team_id or group_id defined.")
            raise ValueError



import json
import logging
from httpx import Response

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    GlobalUser,
)
from .api import Api


class Admin:
    """The class includes all necessary methods to access the Grafana admin API endpoints. Be aware that all functionalities inside the class only working with basic authentication (username and password) and that the authenticated user is a Grafana Admin.

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_settings(self) -> dict:
        """The method includes a functionality to get the settings

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding settings
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/settings",
        )

        if api_call == dict() or api_call.get("DEFAULT") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def update_settings(self, updates: dict = None, removals: dict = None):
        """The method includes a functionality to update the settings. Be aware that the functionality is a Grafana v8.0+ feature and you can find detailed information about the dict values here: https://grafana.com/docs/grafana/latest/developers/http_api/admin/#update-settings

        Args:
            updates (dict): Specify the updates object
            removals (dict): Specify the removals object

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if updates is not None or removals is not None:
            settings_update: dict = dict()

            if updates is not None:
                settings_update.update(dict({"updates": updates}))

            if removals is not None:
                settings_update.update(dict({"removals": removals}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/settings",
                RequestsMethods.PUT,
                json.dumps(settings_update),
            )

            if api_call.get("message") != "Settings updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the corresponding settings.")
        else:
            logging.error("There is no updates or removals defined.")
            raise ValueError

    def get_stats(self) -> dict:
        """The method includes a functionality to get the admin statistics

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding statistics
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/stats",
        )

        if api_call == dict() or api_call.get("orgs") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_preview_report(self) -> dict:
        """The method includes a functionality to get a preview report

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the preview report
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/usage-report-preview",
        )

        if api_call == dict() or api_call.get("version") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def create_global_user(self, user: GlobalUser) -> int:
        """The method includes a functionality to create a global user

        Args:
            user (GlobalUser): Specify the global user object

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (int): Returns the corresponding user id
        """

        if (
            user is not None
            and len(user.name) != 0
            and len(user.email) != 0
            and len(user.login) != 0
            and len(user.password) != 0
        ):
            user_object: dict = dict(
                {
                    "name": user.name,
                    "email": user.email,
                    "login": user.login,
                    "password": user.password,
                }
            )

            if user.org_id is not None:
                user_object.update(dict({"OrgId": user.org_id}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users",
                RequestsMethods.POST,
                json.dumps(user_object),
            )

            if api_call.get("message") != "User created":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("id"))
        else:
            logging.error("There is no name, email, login or password defined.")
            raise ValueError

    def update_user_password(self, id: int, password: str):
        """The method includes a functionality to update the global user password

        Args:
            id (int): Specify the user id
            password (str): Specify the user new password

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and len(password) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}/password",
                RequestsMethods.PUT,
                json.dumps(dict({"password": password})),
            )

            if api_call.get("message") != "User password updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully updated the corresponding user password."
                )
        else:
            logging.error("There is no id or password defined.")
            raise ValueError

    def update_user_permissions(self, id: int, is_grafana_admin: bool = None):
        """The method includes a functionality to update the global user permissions

        Args:
            id (int): Specify the user id
            is_grafana_admin (bool): Specify if the user is admin or not

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and is_grafana_admin is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}/permissions",
                RequestsMethods.PUT,
                json.dumps(dict({"isGrafanaAdmin": is_grafana_admin})),
            )

            if api_call.get("message") != "User permissions updated":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully updated the corresponding user permissions."
                )
        else:
            logging.error("There is no id or is_grafana_admin defined.")
            raise ValueError

    def delete_global_user(self, id: int):
        """The method includes a functionality to delete a global user

        Args:
            id (int): Specify the user id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "User deleted":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the corresponding user.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def pause_all_alerts(self):
        """The method includes a functionality to pause all alerts

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/pause-all-alerts",
            RequestsMethods.POST,
            json.dumps(dict({"paused": True})),
        )

        if api_call.get("state") != "Paused":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully paused all alerts.")

    def unpause_all_alerts(self):
        """The method includes a functionality to unpause all alerts

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/pause-all-alerts",
            RequestsMethods.POST,
            json.dumps(dict({"paused": False})),
        )

        if api_call.get("state") != "Unpaused":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully unpaused all alerts.")

    def get_user_auth_token(self, id: int) -> list:
        """The method includes a functionality to get the corresponding user auth token

        Args:
            id (int): Specify the user id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the corresponding user auth tokens
        """

        if id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}/auth-tokens",
            )

            if api_call == list() or api_call[0].get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def revoke_user_auth_token(self, id: int, auth_token_id: int):
        """The method includes a functionality to get the corresponding user auth token

        Args:
            id (int): Specify the user id
            auth_token_id (int): Specify the auth token id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and auth_token_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}/revoke-auth-token",
                RequestsMethods.POST,
                json.dumps(dict({"authTokenId": auth_token_id})),
            )

            if api_call.get("message") != "User auth token revoked":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully revoked the corresponding user token.")
        else:
            logging.error("There is no id or auth_token_id defined.")
            raise ValueError

    def logout_user(self, id: int):
        """The method includes a functionality to log out the corresponding user

        Args:
            id (int): Specify the user id

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ADMIN.value}/users/{id}/logout",
                RequestsMethods.POST,
                json.dumps(dict()),
            )

            if api_call.get("message") != "User auth token revoked":
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully logout the corresponding user.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def reload_dashboards_provisioning_configuration(self):
        """The method includes a functionality to reload the dashboards provisioning configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/provisioning/dashboards/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "Dashboards config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the dashboards.")

    def reload_datasources_provisioning_configuration(self):
        """The method includes a functionality to reload the datasources provisioning configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/provisioning/datasources/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "Datasources config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the datasources.")

    def reload_plugins_provisioning_configuration(self):
        """The method includes a functionality to reload the plugins provisioning configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/provisioning/plugins/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "Plugins config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the plugins.")

    def reload_notifications_provisioning_configuration(self):
        """The method includes a functionality to reload the notifications provisioning configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/provisioning/notifications/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "Notifications config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the notifications.")

    def reload_access_controls_provisioning_configuration(self):
        """The method includes a functionality to reload the access-controls provisioning configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/provisioning/access-control/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "Accesscontrol config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the accesscontrol.")

    def reload_ldap_configuration(self):
        """The method includes a functionality to reload the ldap configuration

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/ldap/reload",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "LDAP config reloaded":
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully reloaded the config of the ldap.")

    def rotate_data_encryption_keys(self):
        """The method includes a functionality to rotate the data encryption keys

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: Response = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ADMIN.value}/encryption/rotate-data-keys",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.status_code != 204:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully rotated the data keys.")



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class Preferences:
    """The class includes all necessary methods to access the Grafana preferences API endpoints.

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_current_user_preferences(
        self,
    ) -> dict:
        """The method includes a functionality to get the current user preferences

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the current user preferences
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.USER_PREFERENCES.value,
        )

        if isinstance(api_call, dict) is False or api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def update_current_user_preferences(
        self,
        theme: str = None,
        timezone: str = None,
        home_dashboard_id: int = 0,
        home_dashboard_uid: str = None,
    ):
        """The method includes a functionality to update the current user preferences

        Args:
            theme (str): Specify the theme e.g. light, dark, or an empty string for the default theme (default None)
            home_dashboard_id (int): Specify the dashboard id of the home folder (default 0)
            home_dashboard_uid (str): Specify the home team dashboard by uid (default None)
            timezone (str): Specify the timezone e.g. utc, browser, or an empty string for the default (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            theme is not None
            or (home_dashboard_id != 0 or home_dashboard_uid is not None)
            or timezone is not None
        ):
            modified_values: dict = dict()

            if theme is not None:
                modified_values.update(dict({"theme": theme}))

            if home_dashboard_id != 0 and isinstance(home_dashboard_id, int):
                modified_values.update(dict({"homeDashboardId": home_dashboard_id}))
            else:
                modified_values.update({"homeDashboardUID": home_dashboard_uid})

            if timezone is not None:
                modified_values.update(dict({"timezone": timezone}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.USER_PREFERENCES.value,
                RequestsMethods.PATCH,
                json.dumps(modified_values),
            )

            if api_call.get("message") != "Preferences updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the current user preferences.")
        else:
            logging.error("There is no updated value defined.")
            raise ValueError

    def get_current_org_preferences(
        self,
    ) -> dict:
        """The method includes a functionality to get the current org preferences

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the current user preferences
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.ORG_PREFERENCES.value,
        )

        if isinstance(api_call, dict) is False or api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def update_current_org_preferences(
        self,
        theme: str = None,
        home_dashboard_id: int = 0,
        home_dashboard_uid: str = None,
        timezone: str = None,
    ):
        """The method includes a functionality to update the current org preferences

        Args:
            theme (str): Specify the theme e.g. light, dark, or an empty string for the default theme (default None)
            home_dashboard_id (int): Specify the dashboard id of the home folder (default 0)
            home_dashboard_uid (str): Specify the home team dashboard by uid (default None)
            timezone (str): Specify the timezone e.g. utc, browser, or an empty string for the default (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            theme is not None
            or (home_dashboard_id != 0 or home_dashboard_uid is not None)
            or timezone is not None
        ):
            modified_values: dict = dict()

            if theme is not None:
                modified_values.update(dict({"theme": theme}))

            if home_dashboard_id != 0 and isinstance(home_dashboard_id, int):
                modified_values.update(dict({"homeDashboardId": home_dashboard_id}))
            else:
                modified_values.update({"homeDashboardUID": home_dashboard_uid})

            if timezone is not None:
                modified_values.update(dict({"timezone": timezone}))

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.ORG_PREFERENCES.value,
                RequestsMethods.PATCH,
                json.dumps(modified_values),
            )

            if api_call.get("message") != "Preferences updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the current org preferences.")
        else:
            logging.error("There is no updated value defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
)
from .api import Api


class Snapshot:
    """The class includes all necessary methods to access the Grafana snapshot API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def create_new_snapshot(
        self,
        dashboard_json: dict,
        name: str = None,
        expires: int = None,
        external: bool = False,
        key: str = None,
        delete_key: str = None,
    ) -> dict:
        """The method includes a functionality to create the specified dashboard snapshot

        Args:
            dashboard_json (dict): Specify the dashboard_json of the dashboard
            name (str): Specify the optional name of the dashboard snapshot
            expires (int): Specify the optional expiry time as seconds of the dashboard snapshot. 3600 is 1 hour, 86400 is 1 day (default never expired)
            external (bool): Specify the optional external server rather than locally (default False)
            key (str): Specify the optional unique key. Required if external is true.
            delete_key (str): Specify the optional unique key used to delete the snapshot. It is different from the key so that only the creator can delete the snapshot. Required if external is true.

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the snapshot information of the dashboard
        """

        if dashboard_json != dict():
            if external:
                if (key is None or len(key) == 0) and (
                    delete_key is None or len(delete_key) == 0
                ):
                    logging.error(
                        "It's necessary that you define the key and the delete_key, if you use the external snapshot "
                        "opportunity. "
                    )
                    raise ValueError

            snapshot_json: dict = {
                "dashboard": dashboard_json,
                "name": name,
                "expires": expires,
                "external": external,
                "key": key,
                "deleteKey": delete_key,
            }

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.SNAPSHOTS.value,
                RequestsMethods.POST,
                json.dumps(snapshot_json),
            )

            if api_call == dict() or (
                api_call.get("id") is None and api_call.get("key") is None
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard_json defined.")
            raise ValueError

    def get_snapshots(self) -> list:
        """The method includes a functionality to list all dashboard snapshots

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all dashboard snapshots
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.DASHBOARD_SNAPSHOTS.value,
        )

        if api_call == list() or api_call[0].get("name") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_snapshot_by_key(self, key: str) -> dict:
        """The method includes a functionality to get a specific dashboard snapshot by the key

        Args:
            key (str): Specify the key of the dashboard snapshot

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a specific dashboard snapshot
        """

        if len(key) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SNAPSHOTS.value}/{key}",
            )

            if api_call == dict() or api_call.get("dashboard").get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no key defined.")
            raise ValueError

    def delete_snapshot_by_key(self, key: str):
        """The method includes a functionality to delete a specific dashboard snapshot by the key

        Args:
            key (str): Specify the key of the dashboard snapshot

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(key) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SNAPSHOTS.value}/{key}", RequestsMethods.DELETE
            )

            if (
                api_call.get("message")
                != "Snapshot deleted. It might take an hour before it's cleared from any CDN caches."
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully destroyed the dashboard snapshot.")
        else:
            logging.error("There is no key defined.")
            raise ValueError

    def delete_snapshot_by_delete_key(self, delete_key: str):
        """The method includes a functionality to delete a specific dashboard snapshot by the delete_key

        Args:
            delete_key (str): Specify the delete_key of the dashboard snapshot

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(delete_key) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SNAPSHOTS.value}-delete/{delete_key}",
            )

            if (
                api_call.get("message")
                != "Snapshot deleted. It might take an hour before it's cleared from any CDN caches."
            ):
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully destroyed the dashboard snapshot.")
        else:
            logging.error("There is no delete_key defined.")
            raise ValueError



import json
import logging

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    DatasourceCache,
    DatasourcePermission,
)
from .api import Api


class Datasource:
    """The class includes all necessary methods to access the Grafana datasource API endpoints. It's required that the API token got the corresponding datasource access rights. Please check the used methods docstring for the necessary access rights

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_all_datasources(self) -> list:
        """The method includes a functionality to get all datasources

        Required Permissions:
            Action: datasources:read
            Scope: datasources:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the list of all datasources
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.DATASOURCES.value,
            RequestsMethods.GET,
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_datasource_by_id(self, datasource_id: int) -> dict:
        """The method includes a functionality to get the datasource specified by the datasource id

        Args:
            datasource_id (int): Specify the id of the datasource

        Required Permissions:
            Action: datasources:read
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if datasource_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_id defined.")
            raise ValueError

    def get_datasource_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to get the datasource specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources:read
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<uid>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{uid}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_datasource_by_name(self, name: str) -> dict:
        """The method includes a functionality to get the datasource specified by the datasource name

        Args:
            name (str): Specify the name of the datasource

        Required Permissions:
            Action: datasources:read
            Scope: [datasources:*, datasources:name:*, datasources:name:<name>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/name/{name}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def get_datasource_id_by_name(self, name: str) -> int:
        """The method includes a functionality to get the datasource id specified by the datasource name

        Args:
            name (str): Specify the name of the datasource

        Required Permissions:
            Action: datasources:read
            Scope: [datasources:*, datasources:name:*, datasources:name:<name>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (int): Returns a datasource id
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/id/{name}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("id"))
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def create_datasource(self, data_source: dict):
        """The method includes a functionality to create a datasource specified by the datasource as dict

        Args:
            data_source (dict): Specify the datasource as dict

        Required Permissions:
            Action: datasources:create

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if data_source != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.DATASOURCES.value,
                RequestsMethods.POST,
                json.dumps(data_source),
            )

            if api_call.get("message") != "Datasource added":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created a datasource.")
        else:
            logging.error("There is no data_source defined.")
            raise ValueError

    def update_datasource(self, datasource_id: int, data_source: dict):
        """The method includes a functionality to update a datasource specified by the datasource as dict and the datasource id

        Args:
            datasource_id (int): Specify the id of the datasource
            data_source (dict): Specify the datasource as dict

        Required Permissions:
            Action: datasources:write
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0 and data_source != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}",
                RequestsMethods.PUT,
                json.dumps(data_source),
            )

            if api_call.get("message") != "Datasource updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated a datasource.")
        else:
            logging.error("There is no datasource_id or data_source defined.")
            raise ValueError

    def delete_datasource_by_id(self, datasource_id: int):
        """The method includes a functionality to delete a datasource specified by the datasource id

        Args:
            datasource_id (int): Specify the id of the datasource

        Required Permissions:
            Action: datasources:delete
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Data source deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a datasource.")
        else:
            logging.error("There is no datasource_id defined.")
            raise ValueError

    def delete_datasource_by_uid(self, uid: str):
        """The method includes a functionality to delete a datasource specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources:delete
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<uid>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/uid/{uid}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Data source deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a datasource.")
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def delete_datasource_by_name(self, name: str):
        """The method includes a functionality to delete a datasource specified by the datasource name

        Args:
            name (str): Specify the name of the datasource

        Required Permissions:
            Action: datasources:delete
            Scope: [datasources:*, datasources:name:*, datasources:name:<name>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/name/{name}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Data source deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted a datasource.")
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def query_datasource_by_id(
        self, time: str, to: str, datasource_queries: list
    ) -> dict:
        """The method includes a functionality to execute a queries inside the datasource itself specified by the datasource id

        Args:
            from (str): Specify the name of the absolute in epoch timestamps in milliseconds or relative using Grafana time units. For example, now-1h
            to (str): Specify the name of the absolute in epoch timestamps in milliseconds or relative using Grafana time units. For example, now-1h
            datasource_queries (list): Specify a list of execution queries based on the DatasourceQuery class

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the result of the specified query
        """

        if len(time) != 0 and len(to) != 0 and datasource_queries != list():
            datasource_queries_json_list: list = list()

            for datasource_query in datasource_queries:
                datasource_query_json_dict: dict = dict(
                    {
                        "refId": datasource_query.ref_id,
                        "intervalMs": datasource_query.interval_ms,
                        "maxDataPoints": datasource_query.max_data_points,
                        "datasourceId": datasource_query.datasource_id,
                        "rawSql": datasource_query.raw_sql,
                        "format": datasource_query.output_format,
                    }
                )
                datasource_queries_json_list.append(datasource_query_json_dict)

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.DATASOURCE_QUERY.value,
                RequestsMethods.POST,
                json.dumps(datasource_queries_json_list),
            )

            if api_call == dict() or api_call.get("results") == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call.get("results")
        else:
            logging.error("There is no time, to or datasource_queries defined.")
            raise ValueError


class DatasourcePermissions:
    """The class includes all necessary methods to access the Grafana datasource permissions API endpoints. It's required that the API token got the corresponding datasource access rights. Please check the used methods docstring for the necessary access rights

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_datasource_permissions_by_uid(self, uid: str) -> list:
        """The method includes a functionality to get the datasource permissions specified by the datasource uid. The functionality is a Grafana ENTERPRISE feature

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources.permissions:read
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the datasource permissions
        """

        if len(uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCE_PERMISSIONS.value}/{uid}",
            )

            status_code: int = (
                api_call[0].get("status")
                if isinstance(api_call, list)
                else api_call.get("status")
            )

            datasource_permissions_status_dict: dict = dict(
                {
                    401: "Unauthorized.",
                    403: "Access Denied.",
                    500: "Internal error.",
                }
            )

            if status_code == 200:
                return api_call
            elif 401 <= status_code <= 500:
                logging.error(datasource_permissions_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def update_datasource_user_access_by_uid(
        self, uid: str, id: int, datasource_user_permission: DatasourcePermission
    ):
        """The method includes a functionality to update the datasource permission specified by the datasource uid and the user id. The functionality is a Grafana ENTERPRISE feature

        Args:
            uid (str): Specify the uid of the datasource
            id (int): Specify the id of the user
            datasource_user_permission (DatasourcePermission): Specify the datasource user permission

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            len(uid) != 0
            and id != 0
            and datasource_user_permission.permission is not None
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCE_PERMISSIONS.value}/{uid}/users/{id}",
                RequestsMethods.POST,
                json.dumps({"permission": datasource_user_permission.permission}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            datasource_permissions_status_dict: dict = dict(
                {
                    400: "Permission cannot be added, see response body for details.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Permission updated":
                logging.info("You successfully updated the datasource user permission.")
            elif status_code == 200 and api_call.get("message") == "Permission removed":
                logging.info("You successfully removed the datasource user permission.")
            elif 400 == status_code:
                logging.error(
                    f"{datasource_permissions_status_dict.get(status_code)} | Response: {api_call}"
                )
                raise Exception
            elif 401 <= status_code <= 403:
                logging.error(datasource_permissions_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error(
                "There is no uid, id or datasource_user_permission object defined."
            )
            raise ValueError

    def update_datasource_team_access_by_uid(
        self, uid: str, id: int, datasource_team_permission: DatasourcePermission
    ):
        """The method includes a functionality to update the datasource permission specified by the datasource uid and the team id. The functionality is a Grafana ENTERPRISE feature

        Args:
            uid (str): Specify the uid of the datasource
            id (int): Specify the id of the team
            datasource_team_permission (DatasourcePermission): Specify the datasource team permission

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            len(uid) != 0
            and id != 0
            and datasource_team_permission.permission is not None
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCE_PERMISSIONS.value}/{uid}/teams/{id}",
                RequestsMethods.POST,
                json.dumps({"permission": datasource_team_permission.permission}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            datasource_permissions_status_dict: dict = dict(
                {
                    400: "Permission cannot be added, see response body for details.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Permission updated":
                logging.info("You successfully updated the datasource team permission.")
            elif status_code == 200 and api_call.get("message") == "Permission removed":
                logging.info("You successfully removed the datasource team permission.")
            elif 400 == status_code:
                logging.error(
                    f"{datasource_permissions_status_dict.get(status_code)} | Response: {api_call}"
                )
                raise Exception
            elif 401 <= status_code <= 403:
                logging.error(datasource_permissions_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error(
                "There is no uid, id or datasource_team_permission object defined."
            )
            raise ValueError

    def update_datasource_basic_role_access_by_uid(
        self,
        uid: str,
        build_in_role_name: str,
        datasource_team_permission: DatasourcePermission,
    ):
        """The method includes a functionality to update the datasource permission specified by the datasource uid and the build in role name. The functionality is a Grafana ENTERPRISE feature

        Args:
            uid (str): Specify the uid of the datasource
            build_in_role_name (str): Specify the build in role name
            datasource_team_permission (DatasourcePermission): Specify the datasource team permission

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:uid:*, datasources:uid:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            len(uid) != 0
            and len(build_in_role_name) != 0
            and datasource_team_permission.permission is not None
        ):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCE_PERMISSIONS.value}/{uid}/builtInRoles/{build_in_role_name}",
                RequestsMethods.POST,
                json.dumps({"permission": datasource_team_permission.permission}),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            datasource_permissions_status_dict: dict = dict(
                {
                    400: "Permission cannot be added, see response body for details.",
                    401: "Unauthorized.",
                    403: "Access Denied.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Permission updated":
                logging.info(
                    "You successfully updated the datasource build in role name permission."
                )
            elif status_code == 200 and api_call.get("message") == "Permission removed":
                logging.info(
                    "You successfully removed the datasource build in role name permission."
                )
            elif 400 == status_code:
                logging.error(
                    f"{datasource_permissions_status_dict.get(status_code)} | Response: {api_call}"
                )
                raise Exception
            elif 401 <= status_code <= 403:
                logging.error(datasource_permissions_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error(
                "There is no uid, build_in_role_name or datasource_team_permission object defined."
            )
            raise ValueError


class DatasourceLegacyPermissions:
    """The class includes all necessary methods to access the Grafana legacy datasource permissions API endpoints. It's required that the API token got the corresponding datasource access rights. Please check the used methods docstring for the necessary access rights

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def enable_datasource_permissions(self, datasource_id: int):
        """The method includes a functionality to enable datasource permissions specified by the datasource id. The functionality is a Grafana ENTERPRISE feature

        Args:
            datasource_id (int): Specify the id of the datasource

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}/enable-permissions",
                RequestsMethods.POST,
                json.dumps({}),
            )

            if api_call.get("message") != "Datasource permissions enabled":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully enabled the datasource permissions.")
        else:
            logging.error("There is no datasource_id defined.")
            raise ValueError

    def disable_datasource_permissions(self, datasource_id: int):
        """The method includes a functionality to disable datasource permissions specified by the datasource id. The functionality is a Grafana ENTERPRISE feature

        Args:
            datasource_id (int): Specify the id of the datasource

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}/disable-permissions",
                RequestsMethods.POST,
                json.dumps({}),
            )

            if api_call.get("message") != "Datasource permissions disabled":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully disabled the datasource permissions.")
        else:
            logging.error("There is no datasource_id defined.")
            raise ValueError

    def get_datasource_permissions(self, datasource_id: int) -> dict:
        """The method includes a functionality to get the datasource permissions specified by the datasource id. The functionality is a Grafana ENTERPRISE feature

        Args:
            datasource_id (int): Specify the id of the datasource

        Required Permissions:
            Action: datasources.permissions:read
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the datasource permissions
        """

        if datasource_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}/permissions",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("datasourceId") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no datasource_id defined.")
            raise ValueError

    def add_datasource_permissions(
        self, datasource_id: int, datasource_permission: dict
    ):
        """The method includes a functionality to add datasource permission specified by the datasource id and the datasource permission dict. The functionality is a Grafana ENTERPRISE feature

        Args:
            datasource_id (int): Specify the id of the datasource
            datasource_permission (dict): Specify the permission of the datasource

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0 and datasource_permission != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}/permissions",
                RequestsMethods.POST,
                json.dumps(datasource_permission),
            )

            if api_call.get("message") != "Datasource permission added":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully added a datasource permission.")
        else:
            logging.error("There is no datasource_id or datasource_permission defined.")
            raise ValueError

    def delete_datasource_permissions(self, datasource_id: int, permission_id: int):
        """The method includes a functionality to delete datasource permission specified by the datasource id and the permission id. The functionality is a Grafana ENTERPRISE feature

        Args:
            datasource_id (int): Specify the id of the datasource
            permission_id (id): Specify the permission id

        Required Permissions:
            Action: datasources.permissions:write
            Scope: [datasources:*, datasources:id:*, datasources:id:<id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if datasource_id != 0 and permission_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{datasource_id}/permissions/{permission_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Datasource permission removed":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully removed a datasource permission.")
        else:
            logging.error("There is no datasource_id or permission_id defined.")
            raise ValueError


class DatasourceQueryResourceCaching:
    """The class includes all necessary methods to access the Grafana datasource query and resource caching API endpoints. It's required that the API token got the corresponding datasource access rights. Please check the used methods docstring for the necessary access rights. The functionality is a Grafana ENTERPRISE feature

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_datasource_cache(self, uid: str) -> dict:
        """The method includes a functionality to get the datasource cache config specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources.caching:write
            Scope: datasources:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{uid}/cache",
            )

            if api_call == dict() or api_call.get("dataSourceID") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def enable_datasource_cache(self, uid: str) -> dict:
        """The method includes a functionality to enable the datasource cache specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources.caching:read
            Scope: datasources:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{uid}/cache/enable",
                RequestsMethods.POST,
            )

            if api_call == dict() or api_call.get("dataSourceID") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def disable_datasource_cache(self, uid: str) -> dict:
        """The method includes a functionality to disable the datasource cache specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources.caching:write
            Scope: datasources:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{uid}/cache/disable",
                RequestsMethods.POST,
            )

            if api_call == dict() or api_call.get("dataSourceID") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def clean_datasource_cache(self, uid: str) -> dict:
        """The method includes a functionality to clean the datasource cache of all data sources with caching enabled. The uid of the datasource will only be used to return the configuration for that data source

        Args:
            uid (str): Specify the uid of the datasource

        Required Permissions:
            Action: datasources.caching:write
            Scope: datasources:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{uid}/cache/clean",
                RequestsMethods.POST,
            )

            if api_call == dict() or api_call.get("dataSourceID") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def update_datasource_cache(
        self, uid: str, datasource_cache: DatasourceCache
    ) -> dict:
        """The method includes a functionality to update the datasource cache specified by the datasource uid

        Args:
            uid (str): Specify the uid of the datasource
            datasource_cache (DatasourceCache): Specif the datasource cache object

        Required Permissions:
            Action: datasources.caching:write
            Scope: datasources:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a datasource
        """

        if (
            len(uid) != 0
            and datasource_cache is not None
            and datasource_cache.datasource_id != 0
            and len(datasource_cache.datasource_uid) != 0
            and datasource_cache.enabled is not None
            and datasource_cache.use_default_ttl is not None
            and datasource_cache.ttl_queries_ms != 0
            and datasource_cache.ttl_resources_ms != 0
        ):
            datasource_cache_object: dict = dict(
                {
                    "dataSourceID": datasource_cache.datasource_id,
                    "dataSourceUID": datasource_cache.datasource_uid,
                    "enabled": datasource_cache.enabled,
                    "useDefaultTTL": datasource_cache.use_default_ttl,
                    "ttlQueriesMs": datasource_cache.ttl_queries_ms,
                    "ttlResourcesMs": datasource_cache.ttl_resources_ms,
                }
            )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.DATASOURCES.value}/{uid}/cache",
                RequestsMethods.POST,
                json.dumps(datasource_cache_object),
            )

            if api_call == dict() or api_call.get("dataSourceID") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "There is no uid or the right datasource_cache object defined."
            )
            raise ValueError



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class Organisation:
    """The class includes all necessary methods to access the Grafana organisation API endpoint

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_current_organization(self) -> dict:
        """The method includes a functionality to get the current organization

        Required Permissions:
            Action: orgs:read
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the current organization
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.ORGANISATION.value
        )

        if api_call == dict() or api_call.get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_all_users_by_the_current_organization(self) -> list:
        """The method includes a functionality to get all users from the current organization

        Required Permissions:
            Action: org.users:read
            Scope: users:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the users of the current organization
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ORGANISATION.value}/users"
        )

        if api_call == list() or api_call[0].get("orgId") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_all_users_by_the_current_organization_lookup(self) -> list:
        """The method includes a functionality to get the lookup information of all users from the current organization

        Required Permissions:
            Action: org.users:read
            Scope: users:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the users of the current organization
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ORGANISATION.value}/users/lookup"
        )

        if api_call == list() or api_call[0].get("userId") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def update_organization_user_role_by_user_id(self, user_id: int, role: str):
        """The method includes a functionality to update the current organization user by the user id

        Args:
            user_id (int): Specify the id of the user
            role (str): Specify the role of the user

        Required Permissions:
            Action: org.users.role:update
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATION.value}/users/{user_id}",
                RequestsMethods.PATCH,
                json.dumps(dict({"role": role})),
            )

            if api_call.get("message") != "Organization user updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the organization user.")
        else:
            logging.error("There is no user_id or dict defined.")
            raise ValueError

    def delete_organization_user_by_user_id(self, user_id: int):
        """The method includes a functionality to delete the current organization user by the user id

        Args:
            user_id (int): Specify the id of the user

        Required Permissions:
            Action: org.users:remove
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if user_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATION.value}/users/{user_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "User removed from organization":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully removed the organization user.")
        else:
            logging.error("There is no user_id defined.")
            raise ValueError

    def update_current_organization(self, name: str):
        """The method includes a functionality to update the current organization

        Args:
            name (str): Specify the new name of the current organization

        Required Permissions:
            Action: orgs:write
            Scope: N/A

        Raises
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.ORGANISATION.value,
                RequestsMethods.PUT,
                json.dumps(dict({"name": name})),
            )

            if api_call.get("message") != "Organization updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the organization.")
        else:
            logging.error("There is no role_name defined.")
            raise ValueError

    def add_new_user_to_current_organization(
        self, login_or_email: str, role: str
    ) -> int:
        """The method includes a functionality to add a new user to the current organization

        Args:
            login_or_email (str): Specify the added user
            role (str): Specify the added role for the user

        Required Permissions:
            Action: org.users:add
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            user_id (int): Returns the id of the created user
        """

        if len(login_or_email) != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATION.value}/users",
                RequestsMethods.POST,
                json.dumps(dict({"loginOrEmail": login_or_email, "role": role})),
            )

            if api_call.get("message") != "User added to organization":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call.get("userId")
        else:
            logging.error("There is no user_role defined.")
            raise ValueError


class OrganisationAdmin:
    """The class includes all necessary methods to access the Grafana organisation Admin API endpoint. Be aware that all functionalities inside the class only working with basic authentication (username and password)

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_organization_by_id(self, org_id: int) -> dict:
        """The method includes a functionality to get an organization by the id

        Args:
            org_id (int): Specify the organization id

        Required Permissions:
            Action: orgs:read
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the organization as dict
        """

        if org_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}"
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def get_organization_by_name(self, name: str) -> dict:
        """The method includes a functionality to get an organization by the name

        Args:
            name (str): Specify the organization name

        Required Permissions:
            Action: orgs:read
            Scope: N/A
            Note: Needs to be assigned globally.

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the organization as dict
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/name/{name}"
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def get_organizations(self) -> list:
        """The method includes a functionality to get all organizations

        Required Permissions:
            Action: orgs:read
            Scope: N/A
            Note: Needs to be assigned globally.

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
             api_call (list): Returns all organizations as list
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.ORGANISATIONS.value
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def create_organization(self, name: str) -> int:
        """The method includes a functionality to create an organization

        Args:
            name (str): Specify the organization name

        Required Permissions:
            Action: orgs:create
            Scope: N/A
            Note: Needs to be assigned globally.

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            org_id (int): Returns the id of the created organization
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.ORGANISATIONS.value,
                RequestsMethods.POST,
                json.dumps(dict({"name": name})),
            )

            if api_call.get("message") != "Organization created":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return int(api_call.get("orgId"))
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def update_organization(self, org_id: int, name: str):
        """The method includes a functionality to update an organization

        Args:
            org_id (int): Specify the organization id
            name (str): Specify the organization name

        Required Permissions:
            Action: orgs:write
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if org_id != 0 and len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}",
                RequestsMethods.PUT,
                json.dumps(dict({"name": name})),
            )

            if api_call.get("message") != "Organization updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the organization.")
        else:
            logging.error("There is no org_id or name defined.")
            raise ValueError

    def delete_organization(self, org_id: int):
        """The method includes a functionality to delete an organization

        Args:
            org_id (int): Specify the organization id

        Required Permissions:
            Action: orgs:delete
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if org_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Organization deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the organization.")
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def get_organization_users(self, org_id: int) -> list:
        """The method includes a functionality to get all organization users specified by the organization id

        Args:
            org_id (int): Specify the organization id

        Required Permissions:
            Action: org.users:read
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all organization users as list
        """

        if org_id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}/users"
            )

            if api_call == list() or api_call[0].get("orgId") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no org_id defined.")
            raise ValueError

    def add_organization_user(self, org_id: int, login_or_email: str, role: str) -> int:
        """The method includes a functionality to add a user to an organization

        Args:
            org_id (int): Specify the organization id
            login_or_email (str): Specify the added user
            role (str): Specify the added role for the user

        Required Permissions:
            Action: org.users:add
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            user_id (int): Returns the added user id
        """

        if org_id != 0 and len(login_or_email) != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}/users",
                RequestsMethods.POST,
                json.dumps(dict({"loginOrEmail": login_or_email, "role": role})),
            )

            if api_call.get("message") != "User added to organization":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call.get("userId")
        else:
            logging.error("There is no org_id, login_or_email or role defined.")
            raise ValueError

    def update_organization_user(self, org_id: int, user_id: int, role: str):
        """The method includes a functionality to update organization user specified by the organization id, the user_id and the role

        Args:
            org_id (int): Specify the organization id
            user_id (int): Specify the user id
            role (str): Specify the added role for the user

        Required Permissions:
            Action: org.users.role:update
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if org_id != 0 and user_id != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}/users/{user_id}",
                RequestsMethods.PATCH,
                json.dumps(dict({"role": role})),
            )

            if api_call.get("message") != "Organization user updated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated user inside the organization.")
        else:
            logging.error("There is no org_id, user_id or role defined.")
            raise ValueError

    def delete_organization_user(self, org_id: int, user_id: int):
        """The method includes a functionality to remove an organization users specified by the organization id and the user id

        Args:
            org_id (int): Specify the organization id
            user_id (int): Specify the user id

        Required Permissions:
            Action: org.users:remove
            Scope: users:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if org_id != 0 and user_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ORGANISATIONS.value}/{org_id}/users/{user_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "User removed from organization":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully removed user from the organization.")
        else:
            logging.error("There is no org_id or user_id defined.")
            raise ValueError



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class Authentication:
    """The class includes all necessary methods to access the Grafana authentication API endpoints.

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_api_tokens(
        self,
        org_id_header: int = None,
    ) -> list:
        """The method includes a functionality to get the corresponding api tokens of the organisation

        Args:
            org_id_header (int): Specify the org id as header to execute call for that specific organisation

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the result of the API token call
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.AUTHENTICATION.value,
            org_id_header=org_id_header,
        )

        if api_call != list() and api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def create_api_token(
        self,
        name: str,
        role: str,
        seconds_to_live: int = 0,
        org_id_header: int = None,
    ) -> dict:
        """The method includes a functionality to create a corresponding api tokens of the organisation specified by the name and role

        Args:
            name (str): Specify the token name
            role (str): Specify the access level/Grafana Role for the token. Can be one of the following values: Viewer, Editor or Admin
            seconds_to_live (int): Specify the seconds of the token expiration in seconds. If it is a positive number an expiration date for the key is set. If it is null, zero or is omitted completely (unless api_key_max_seconds_to_live configuration option is set) the key will never expire (default 0)
            org_id_header (int): Specify the org id as header to execute call for that specific organisation

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the API token object
        """

        if len(name) != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.AUTHENTICATION.value,
                RequestsMethods.POST,
                json.dumps(
                    dict(
                        {
                            "name": name,
                            "role": role,
                            "secondsToLive": seconds_to_live,
                        }
                    )
                ),
                org_id_header=org_id_header,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name or role defined.")
            raise ValueError

    def delete_api_token(
        self,
        token_id: int,
        org_id_header: int = None,
    ):
        """The method includes a functionality to delete a corresponding api token specified by the token id

        Args:
            token_id (int): Specify the token id
            org_id_header (int): Specify the org id as header to execute call for that specific organisation

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if token_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.AUTHENTICATION.value}/{token_id}",
                RequestsMethods.DELETE,
                org_id_header=org_id_header,
            )

            if api_call.get("message") != "API key deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the token.")
        else:
            logging.error("There is no token_id defined.")
            raise ValueError
